/****************************************************************
KPIT Cummins Infosystems Ltd, Pune, India. - 17-Jan-2006.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

*****************************************************************/

#include "iodefine.h"
/* RSKM32C87def.h provides common defines for widely used items. */
#include "rskM32C87def.h"
#include "hwinit.h"
#include "inthandler.h"


/***********************************************************************************
User Program Code Prototypes
***********************************************************************************/
/* These functions are private so their prototypes are defined locally */
void ConfigureOperatingFrequency(void);
void EnablePeripheralModules(void);
void ConfigurePortPins(void);

/***********************************************************************************
User Program Code 
***********************************************************************************/

/***********************************************************************************
Function Name: 	Hardwareinitialise
Description:	Sets up hardware
Parameters: none
Return value: none
***********************************************************************************/

void hw_initialise (void)
{
	//hardware initialisation code here
	
	ConfigureOperatingFrequency();

	ConfigurePortPins();

	EnablePeripheralModules();
}

/***********************************************************************************
End of function HardwareSetup
***********************************************************************************/

/***********************************************************************************
Function Name: 	ConfigureOperatingFrequency
Description:	Sets up operating speed and start sub-clock
Parameters: none
Return value: none
***********************************************************************************/
void ConfigureOperatingFrequency(void)
{
    unsigned int i;
  
	/* set protection register to allow writing to clock control registers*/
    prc0 =1;    
	/* cpu clock selected by cm21 in CM2 (oscillation stop detect reg)*/
    cm07 = 0;   
	/* cpu clock selected by cm17 in CM1 (system clock select reg)*/
    cm21 = 0;   
	/* clock source is main clock*/				
    cm17 = 0;   
	/* set divide for BCLK to divide by 1*/
    mcdb = 0x12;  

/*  PLL initialization - If not using PLL this section should be commented out*/
    
	/* set PLL to multiply by 6 and divide by 2 giving 30MHz
	   when writing PLL control registers write to
       both plc0 and plc1 (addresses 0026 and 0027) using a word write command. 
	   Set divide ratio with the PLL off*/
	*(unsigned int*)&plc0 = 0x0253; 
    asm("nop");

	/* turn the PLL on */
	*(unsigned int*)&plc0 = 0x02d3; 

	/* allow 20 mSec at 5V for the PLL to stabilize (a timer could be used for this delay) then
	make the PLL the CPU clock source */
    for (i=0; i<0x4000;i++);  
    cm17=1;      

/* End of PLL initialization */
	/*protect clock control registers*/
	prc0=0;		 
	


/* configure 32 kHz clock if not using 32 kHz clock comment this section out */

  /* Start the 32Khz crystal sub clock */
   	/* Unlock CM0 and CM1 and set GPIO to inputs (XCin/XCout)*/
   	prc0 = 1;  	   
   	pd8_7 = 0;	
   	pd8_6 = 0;
   	/* Start the 32KHz crystal -  before using the 32 kHz some delay should be allowed for the oscillator to stabilize */
   	cm04 = 1; 	   

}
/***********************************************************************************
End of function ConfigureOperatingFrequency
***********************************************************************************/

/***********************************************************************************
Function Name: 	EnablePeripheralModules
Description:	Enables Peripheral Modules before use also enables data flash
Parameters: none
Return value: none
***********************************************************************************/
void EnablePeripheralModules(void)
{
	/* All modules are active in the M32C device */


}
/***********************************************************************************
End of function EnablePeripheralModules
***********************************************************************************/

/***********************************************************************************
Function Name: 	ConfigurePortPins
Description:	Configures port pins 
Parameters: none
Return value: none
***********************************************************************************/
void ConfigurePortPins(void)
{
/* Port pins default to inputs. To ensure safe initialisation set the pin states
before changing the data direction registers. This will avoid any unintentional
state changes on the external ports.
Many peripheral modules will override the setting of the port registers. Ensure
that the state is safe for external devices if the internal peripheral module is
disabled or powered down. */

/* General device configuration */
   				

/* Switch Port configuration */
	/* All pins are inputs by default */



/* Configure unused pins as output low */

	p0b=p1b=p2b=p3b=p4b=p5b=p6b=p7b=p9b=p10b=0x0;       
	/* lower half of port 4 has the LEDs on it so keep the LEDs off */
	p4b= 0x0f;
	/* configure direction registers as outputs*/
	pd0b=pd1b=pd2b=pd3b=pd4b=pd5b=pd7b=pd10b = 0xff;
	
	/* to set port9 direction register must first unprotect register do not step or set breakpoints
	between unprotecting register and modifying the direction register */
	prc2=1;
	pd9b=0xff;
	prc2 =0;

	/* port 6 is used by E8 do not modify upper half of port */
	pd6b |= 0x0f;

	/* port 8 has switches, NMI and sub-clock only modify p8_0 and p8_1 */
	p8_0 = p8_1 = 0;
	pd8_0 = pd8_1 = 1;		

 			
}
/***********************************************************************************
End of function ConfigurePortPins
***********************************************************************************/



